/* editor-theme-selector-private.h
 *
 * Copyright 2021 Christian Hergert <chergert@redhat.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define EDITOR_TYPE_THEME_SELECTOR (editor_theme_selector_get_type())

G_DECLARE_FINAL_TYPE (EditorThemeSelector, editor_theme_selector, EDITOR, THEME_SELECTOR, GtkWidget)

GtkWidget   *_editor_theme_selector_new       (void);
const gchar *_editor_theme_selector_get_theme (EditorThemeSelector *self);
void         _editor_theme_selector_set_theme (EditorThemeSelector *self,
                                               const gchar         *theme);

G_END_DECLS
