/* EditorPreferencesWindow */

.org-gnome-TextEditor.preferences frame {
  border-radius: 5px;
  background: @content_view_bg;
}
.org-gnome-TextEditor.preferences list {
  background: transparent;
}
.org-gnome-TextEditor.preferences list row {
  border-bottom: 1px solid alpha(@borders, .4);
}
.org-gnome-TextEditor.preferences list row:last-child {
  border-bottom: none;
}

.org-gnome-TextEditor.preferences .horizontal.linked .radio.text-button {
  padding: 0;
}
.org-gnome-TextEditor.preferences .horizontal.linked .radio.text-button label {
  padding: 4px 12px 4px 12px;
}


/* EditorWindow */

.org-gnome-TextEditor positionlabel label {
  font-feature-settings: "tnum";
}


.org-gnome-TextEditor .empty-state image.dim-label { color: alpha(@theme_fg_color, 0.5); }
.org-gnome-TextEditor .empty-state image:backdrop.dim-label { color: alpha(@theme_unfocused_fg_color, 0.5); }
.org-gnome-TextEditor .empty-state .title { color: alpha(@theme_fg_color, 0.5); }
.org-gnome-TextEditor .empty-state .title:backdrop { color: alpha(@theme_unfocused_fg_color, 0.5); }


/* EditorThemeSelector */

.org-gnome-TextEditor checkbutton.theme-selector radio {
  -gtk-icon-source: none;
  background: none;
  box-shadow: none;
  padding: 15px;
  min-height: 24px;
  min-width: 24px;
  border-width: 3px;
  border-style: solid;
  border-color: transparent;
}
.org-gnome-TextEditor checkbutton.theme-selector radio:checked {
  border-color: @theme_selected_bg_color;
  -gtk-icon-source: -gtk-scaled(-gtk-icontheme("object-select-symbolic"));
}
.org-gnome-TextEditor checkbutton.theme-selector.light radio {
  color: #2e3436;
  background-color: #fff;
}
.org-gnome-TextEditor checkbutton.theme-selector.dark radio {
  color: #fff;
  background-color: #2e3436;
}


/* EditorOpenPopover */

.org-gnome-TextEditor .open-popover listview row {
  border-bottom: 1px solid alpha(@borders, 0.4);
}
.org-gnome-TextEditor .open-popover listview row:last-child {
  border-bottom: 1px solid transparent;
}
.org-gnome-TextEditor .open-popover listview row button {
  padding: 3px;
  margin: 0;
  min-height: 24px;
  min-width: 24px;
}


/* EditorSearchBar */

.org-gnome-TextEditor page searchbar {
  background-image: linear-gradient(shade(@theme_bg_color,1.05), @theme_bg_color);
  padding: 6px;
  border-style: solid;
  border-color: @borders;
  border-left-width: 1px;
  border-right-width: 1px;
  border-bottom-width: 1px;
  border-bottom-left-radius: 5px;
  border-bottom-right-radius: 5px;
  box-shadow: 0 0 3px 1px alpha(shade(@borders,0.8),.35);
  margin: 0 10px 20px 10px;
}
.org-gnome-TextEditor page searchbar:backdrop {
  box-shadow: none;
}
.org-gnome-TextEditor page searchbar button.flat.circular {
  min-width: 24px;
  min-height: 24px;
  margin: 0px;
  padding: 3px;
}

textview.source-map:dir(ltr) {
  border-left: 1px solid alpha(@borders,.3);
}
textview.source-map:dir(rtl) {
  border-right: 1px solid alpha(@borders,.3);
}
